#!/usr/bin/env python3

# +---------------------------------------------------------------------------------+
# | Date:      02/05/2023                                                           |
# | Copyright (c) 2022-2023 by D. Cole, NK7Z                                        |
# | All rights reserved.                                                            |
# +---------------------------------------------------------------------------------+
# | Purpose:                                                                        |
# | A program to pass keyboard data to telnet in realtime, for driving macro        |
# | commands to a Elecraft K4 Amateur Radio.  Assumes a telnet window had focus at  *
# | all times.  Will create a file called IP.txt.  This file is used to hold the IP |
# | of the radio for the next run.  If the file does not exist, it will be Created. |
# +---------------------------------------------------------------------------------+

# +---------------------------------------------------------------------------------+
# |External files used:                                                             |
# |IP.txt                                                                           |
# |Used to remember the users IP address from run to run.                           |
# +---------------------------------------------------------------------------------+

# +---------------------------------------------------------------------------------+
# |                        Bring in all the modules we will be using                |
# +---------------------------------------------------------------------------------+
import telnetlib, sys, time, os.path, datetime, os
from colorama import Fore, Back, Style

# +---------------------------------------------------------------------------------+
# |                             Suppress all errors to console.                     |
# +---------------------------------------------------------------------------------+
# class DevNull:
#   def write(self, msg):
#      pass
# sys.stderr = DevNull()

# +---------------------------------------------------------------------------------+
# |                              Define later used variables.                       |
# +---------------------------------------------------------------------------------+
HOST = ''  # IP of K4.
PORT = 9200  # Port on K4.
TIMEOUT = 5  # Telnet timeout.
exitTrigger = "EXIT"  # Exit trigger to look for.
version = "1.1.10\n"  # Last version.

# +---------------------------------------------------------------------------------+
# |                                  Clear terminal screen                          |
# +---------------------------------------------------------------------------------+
def clearscreen():
    for x in range(80):  # Clear the screen.
        print("\n")

# +---------------------------------------------------------------------------------+
# |                              Print welcome message                              |
# +---------------------------------------------------------------------------------+
def welcome():
    print('Welcome to Genovation-Pad, G-Pad')
    print('Copyright (c) 2022-2023 by D. Cole, NK7Z')
    print('For updates to this software, visit www.nk7z.net')
    print(Fore.LIGHTCYAN_EX + 'Version: ' + version + Fore.LIGHTWHITE_EX)

# +---------------------------------------------------------------------------------+
# |      Get IP address of radio, and save to a file for use later as default.      |
# +---------------------------------------------------------------------------------+
def getip():
    global HOST
    file_path = r'./ip.txt'  # Set path to storage file.
    flag = os.path.isfile(file_path)  # Set flag var for file exists test.
    if flag:  # Does the file exist test.
        f = open("ip.txt")  # Open the IP storage file.
        HOST = f.read().replace("\n", "")  # Read it, and set HOST from it.
        f.close()  # Clean up.
    else:
        print(Fore.LIGHTRED_EX)  # Set text color to red
        print('ATTENTION: No K4 serial number saved!')
        print(Fore.LIGHTWHITE_EX)  # Set text color to yellow.
        print('To find your serial number tap MENU, and using the large knob, scroll to \'Radio Serial Number\'.')
        print(Fore.LIGHTWHITE_EX)  # Return to white...

        HOST = input("Please enter your K4 serial number, include leading zeros: ")  # Get radio IP address from user.
        HOST = 'K4-SN'+HOST+'.local'
        text_file = open(r'ip.txt', 'w')  # Open a file named ip.txt.
        my_string = HOST  # Copy the IP address...
        text_file.write(my_string)  # Save it to the file.
        text_file.close()  # Clean up.

# +---------------------------------------------------------------------------------+
# |             Create telnet object, and make sure radio is on and connected.      |
# +---------------------------------------------------------------------------------+
def createtelnet():
    global tn
    try:
        tn = telnetlib.Telnet(HOST, PORT, TIMEOUT)  # Init telnet library.
        print("Welcome back!  Default IP address found, using IP address:", HOST)
    except:
        print("\nNo connection found at host address: ", HOST)  # Radio not there...
        print("Is the above host address correct?  Is the radio turned on?\n")
        print(
            "INSTRUCTIONS:To find your serial number tap MENU, and using the large knob, scroll to"
            " \'Radio Serial Number\', and restart the program.")
        print("Closing telnet session, goodbye")
        os.remove("ip.txt")  # Just to be safe delete the old ip.txt.
        quit()  # Leave program.
        # Radio is there...
    print("Telnet session to radio initiated using: " + HOST + ', Port: 9200')
    print("\nUse 'ctrl-c', 'exit' or 'EXIT', to terminate program.\n")
    print("Radio is on-line.")
    print("External keypad is on-line.\nExternal keypad ready for use.")
    print("Use \'EXIT\', or \'exit\',  to end session.")

# +---------------------------------------------------------------------------------+
# |                Waiting for a macro to pass to K4, via telnet                    |
# +---------------------------------------------------------------------------------+
def waitformacro():
    global counter
    while True:
        try:
            a = 5  # Preset WHILE test so it never ends.
            counter = 0  # Preset number of macros counter.
            while a > 1:  # Macro input while loop.
                counter = counter + 1  # Keep track of number of macros used today.
                macro = input("Enter macro: ")  # Get user input.
                macro = str.upper(macro)  # Standardize macro to uppercase.
                lastLtr = macro[-1]  # Get last letter to check for malformation.
                if macro == exitTrigger:  # Time to exit program?
                    # a=0
                    sys.exit()
                if lastLtr != ";":  # Did user forget the ";"?
                    print(Fore.LIGHTYELLOW_EX +  # Set bright color to inform user of error.
                          "Ignoring malformed macro, did you forget the ';'?" +  # Warn the user in yellow...
                          Fore.LIGHTWHITE_EX)  # Return to white.
                    macro = ''  # Clear the bad macro.
                macro = bytes(macro, 'utf-8')  # Nope, keep goint in WHILE loop.
                tn.write(macro)  # Send K4 the macro, and start again.
        except:  # User hit exit trigger, tidy up as we leave...
            print("\n\nExit trigger detected, closing program.")  # User wants out of program...
        finally:
            print('Closing telnet session, you entered ' + str(counter - 1) + ' macros this session.\n\n')
            tn.close()
            sys.exit()

# +---------------------------------------------------------------------------------+
# |                            Main calling routine                                 |
# +---------------------------------------------------------------------------------+
clearscreen()  # Scrolls off anything on screen.
welcome()  # Prints the welcome message.
getip()  # Gets IP, or creates file with IP.
createtelnet()  # Creates the telnet object for later use.
waitformacro()  # Waits for macros, checks for malformed macros.
